#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (c) 2021 LG Electronics Inc.
# SPDX-License-Identifier: Apache-2.0

import logging
import os
from pathlib import Path
import pkg_resources
import sys
import platform
from . import constant as constant
from lastversion import lastversion
import coloredlogs


def init_log(log_file, create_file=True, stream_log_level=logging.INFO, file_log_level=logging.DEBUG):

    logger = logging.getLogger(constant.LOGGER_NAME)

    if not logger.hasHandlers():
        logger.setLevel(logging.DEBUG)
        log_dir = os.path.dirname(log_file)
        Path(log_dir).mkdir(parents=True, exist_ok=True)

        coloredlogs.DEFAULT_LOG_FORMAT = '%(message)s'
        coloredlogs.DEFAULT_LOG_LEVEL = stream_log_level
        coloredlogs.install(logger=logger)

        if create_file:
            file_handlder = logging.FileHandler(log_file)
            file_handlder.setLevel(file_log_level)
            file_formatter = logging.Formatter('[%(levelname)7s] %(message)s')
            file_handlder.setFormatter(file_formatter)
            file_handlder.propagate = False
            logger.addHandler(file_handlder)

        logger.propagate = False

    return logger


def init_check_latest_version(pkg_version="", main_package_name=""):

    logger = logging.getLogger(constant.LOGGER_NAME)

    try:
        has_update = lastversion.has_update(repo=main_package_name, at='pip', current_version=pkg_version)
        if has_update:
            logger.info('### Version Info ###')
            logger.warning('Newer version is available : v{}'.format(str(has_update)))
            logger.warning('You can update it with command (\'pip install ' + main_package_name + ' --upgrade\')')
    except TypeError:
        logger.warning('Cannot check the lastest version on PIP')
        logger.warning('You could use already installed version\n')


def init_log_item(main_package_name="", path_to_analyze=""):

    _PYTHON_VERSION = sys.version_info[0]
    _result_log = {
        "Tool Info": main_package_name,
        "Python version": _PYTHON_VERSION,
        "OS": platform.system()+" "+platform.release(),
    }
    if main_package_name != "":
        pkg_version = pkg_resources.get_distribution(main_package_name).version
        init_check_latest_version(pkg_version, main_package_name)
        _result_log["Tool Info"] = main_package_name + " v" + pkg_version
    if path_to_analyze != "":
        _result_log["Path to analyze"] = path_to_analyze

    return _result_log



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (c) 2021 LG Electronics Inc.
# SPDX-License-Identifier: Apache-2.0
import os
from pathlib import Path


def write_txt_file(file_to_create, str_to_write):
    success = True
    error_msg = ""
    try:
        dir_to_create = os.path.dirname(file_to_create)
        Path(dir_to_create).mkdir(parents=True, exist_ok=True)
        f = open(file_to_create, 'w')
        f.write(str_to_write)
        f.close()
    except Exception as ex:
        error_msg = str(ex)
        success = False
    return success, error_msg



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (c) 2021 LG Electronics Inc.
# SPDX-License-Identifier: Apache-2.0
import xlsxwriter
import csv
import time
import logging
import os
import platform
import pandas as pd
from pathlib import Path
import fosslight_util.constant as constant

_HEADER = {'SRC': ['ID', 'Source Name or Path', 'OSS Name',
                   'OSS Version', 'License',  'Download Location',
                   'Homepage', 'Copyright Text', 'Exclude',
                   'Comment'],
           'BIN': ['ID', 'Binary Name', 'OSS Name', 'OSS Version',
                   'License', 'Download Location', 'Homepage',
                   'Copyright Text', 'Exclude', 'Comment'],
           'BIN (Android)': ['ID', 'Binary Name', 'Source Code Path',
                             'NOTICE.html', 'OSS Name', 'OSS Version',
                             'License', 'Download Location', 'Homepage',
                             'Copyright Text', 'Exclude', 'Comment']}
_OUTPUT_FILE_PREFIX = "FOSSLight-Report_"
_EMPTY_ITEM_MSG = "* There is no item"\
                    " to print in FOSSLight-Report.\n"
logger = logging.getLogger(constant.LOGGER_NAME)


def write_excel_and_csv(filename_without_extension, sheet_list, ignore_os=False):
    # sheet_list = {} // Key = Sheet_name, Value = list of [row_items]
    success = True
    error_msg = ""
    success_csv = True
    error_msg_csv = ""

    is_not_null, sheet_list = remove_empty_sheet(sheet_list)

    if is_not_null:
        output_dir = os.path.dirname(filename_without_extension)
        Path(output_dir).mkdir(parents=True, exist_ok=True)

        success, error_msg = write_result_to_excel(filename_without_extension + ".xlsx", sheet_list)

        if ignore_os or platform.system() != "Windows":
            success_csv, error_msg_csv = write_result_to_csv(filename_without_extension + ".csv", sheet_list)
        if not success:
            error_msg = "[Error] Writing excel:" + error_msg
        if not success_csv:
            error_msg += "\n[Error] Writing csv:" + error_msg_csv
    else:
        success = False
        error_msg = _EMPTY_ITEM_MSG

    return (success and success_csv), error_msg


def remove_empty_sheet(sheet_items):
    skip_sheet_name = []
    cnt_sheet_to_print = 0
    final_sheet_to_print = {}
    success = False

    try:
        if sheet_items:
            for sheet_name, sheet_content in sheet_items.items():
                logger.debug("ITEM COUNT:" + str(len(sheet_content)))
                if len(sheet_content) > 0:
                    final_sheet_to_print[sheet_name] = sheet_content
                    cnt_sheet_to_print += 1
                else:
                    skip_sheet_name.append(sheet_name)
            if cnt_sheet_to_print != 0:
                success = True
                if len(skip_sheet_name) > 0:
                    logger.warn("* Empty sheet(not printed):" + str(skip_sheet_name))
    except Exception as ex:
        logger.warn("* Warning:"+str(ex))

    return success, final_sheet_to_print


def write_result_to_csv(output_file, sheet_list):
    success = True
    error_msg = ""
    _header_added = False
    try:
        row_num = 1
        with open(output_file, 'w', newline='') as file:
            writer = csv.writer(file, delimiter='\t')

            for sheet_name, sheet_contents in sheet_list.items():
                if not _header_added:  # Write a header row only once
                    for header_key in _HEADER.keys():
                        if header_key in sheet_name:
                            writer.writerow(_HEADER[header_key])
                            _header_added = True
                            break
                for row_item in sheet_contents:
                    row_item.insert(0, row_num)
                    writer.writerow(row_item)
                    row_num += 1
    except Exception as ex:
        error_msg = str(ex)
        success = False
    return success, error_msg


def write_result_to_excel(out_file_name, sheet_list):
    success = True
    error_msg = ""
    try:
        workbook = xlsxwriter.Workbook(out_file_name)
        for sheet_name, sheet_contents in sheet_list.items():
            selected_header = ""
            for header_key in _HEADER.keys():
                if header_key in sheet_name:
                    selected_header = header_key
                    break
            worksheet = create_worksheet(workbook, sheet_name, selected_header)
            write_result_to_sheet(worksheet, sheet_contents)
        workbook.close()
    except Exception as ex:
        error_msg = str(ex)
        success = False
    return success, error_msg


def write_result_to_sheet(worksheet, sheet_contents):
    row = 1
    for row_item in sheet_contents:
        worksheet.write(row, 0, row)
        for col_num, value in enumerate(row_item):
            worksheet.write(row, col_num + 1, value)
        row += 1


def create_worksheet(workbook, sheet_name, header_key):
    if len(sheet_name) > 31:
        current_time = str(time.time())
        sheet_name = current_time
    worksheet = workbook.add_worksheet(sheet_name)
    if header_key in _HEADER:
        for col_num, value in enumerate(_HEADER[header_key]):
            worksheet.write(0, col_num, value)
    return worksheet


def merge_excels(find_excel_dir, final_out):
    success = True
    error_msg = ""
    _find_extension = '.xlsx'

    try:
        files = os.listdir(find_excel_dir)

        if len([name for name in files if name.endswith(_find_extension)]) > 0:
            writer = pd.ExcelWriter(final_out)

            for file in files:
                if file.endswith(_find_extension):
                    f_short_name = os.path.splitext(
                        file)[0].replace(_OUTPUT_FILE_PREFIX, "")
                    file = os.path.join(find_excel_dir, file)
                    excel_file = pd.ExcelFile(file, engine='openpyxl')

                    for sheet_name in excel_file.sheet_names:
                        df_excel = pd.read_excel(
                            file, sheet_name=sheet_name, engine='openpyxl')
                        df_excel.to_excel(
                            writer, f_short_name + '_' + sheet_name,
                            index=False)
            writer.save()
    except Exception as ex:
        error_msg = str(ex)
        success = False
    return success, error_msg



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Copyright (c) 2021 LG Electronics
# SPDX-License-Identifier: Apache-2.0
from codecs import open
from setuptools import setup, find_packages

with open('README.md', 'r', 'utf-8') as f:
    readme = f.read()

with open('requirements.txt', 'r', 'utf-8') as f:
    required = f.read().splitlines()

if __name__ == "__main__":
    setup(
        name='fosslight_util',
        version='1.0.12',
        package_dir={"": "src"},
        packages=find_packages(where='src'),
        description='FOSSLight Util',
        long_description=readme,
        long_description_content_type='text/markdown',
        license='Apache-2.0',
        author='LG Electronics',
        url='https://github.com/fosslight/fosslight_util',
        download_url='https://github.com/fosslight/fosslight_util',
        classifiers=['License :: OSI Approved :: Apache Software License',
                     "Programming Language :: Python :: 3",
                     "Programming Language :: Python :: 3.6",
                     "Programming Language :: Python :: 3.7",
                     "Programming Language :: Python :: 3.8",
                     "Programming Language :: Python :: 3.9", ],
        install_requires=required
    )



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (c) 2021 LG Electronics Inc.
# SPDX-License-Identifier: Apache-2.0
import threading
import time
from progress.spinner import Spinner


class TimerThread(threading.Thread):
    def __init__(self):
        threading.Thread.__init__(self, name=' Thread')

    def run(self):
        spinner = Spinner('')
        while True:
            time.sleep(1)
            spinner.next()
